/*******************************************************************************
 * Copyright (c) 2011 Michael Ruflin, Andr Locher, Claudia von Bastian.
 * 
 * This file is part of Tatool.
 * 
 * Tatool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published 
 * by the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * Tatool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Tatool. If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package ch.tatool.core.module.statistics;

import java.awt.CardLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.tatool.data.Module;
import ch.tatool.data.DataService;
import ch.tatool.module.ModuleInfoProvider;

/**
 * Abstract implementation of a JFreeChart based module statistics provider.
 * Handles the case where no sessions are available yet as well as the instantiation of the chart.
 * Subclasses are left with implementing the chart-specific logic.
 * 
 * @author Michael Ruflin
 */
public abstract class AbstractChartStatistics extends JPanel implements ModuleInfoProvider {

	private static final long serialVersionUID = -4918844997152380258L;

	Logger logger = LoggerFactory.getLogger(AbstractChartStatistics.class);

    protected static final String NO_DATA_CARD = "NoData";
    protected static final String CHART_CARD = "Chart";
    
    // Card layout to switch between no data label and chart
    protected CardLayout cardLayout;
    protected JLabel noDataLabel;
    protected JPanel chartPanel;
    
    public AbstractChartStatistics() {
        initComponents();
    }
    
    /**
     * Return this as statistics panel
     */
    public JPanel getModuleInfoPanel() {
    	return this;
    }
    
    /**
     * Updates the statistics display.
     */
    public void updateModuleInfo(DataService dataService, Module module) {
        // check the number of sessions
        long sessionCount = dataService.getSessionCount(module, false);
        if (sessionCount < 1) {
        	// display the no data message
        	setDisplayedCard(NO_DATA_CARD);
        } else {
        	// update the chartPanel with a new chart
        	chartPanel = createChart(dataService, module);
        	add(chartPanel, CHART_CARD);
            setDisplayedCard(CHART_CARD);
        }
    }
    
    public void setDisplayedCard(String card) {
    	cardLayout.show(this, card);
    }
    
    /** Initializes the ui components. */
    private void initComponents() {
        // initialize the layout
        cardLayout = new CardLayout();
        setLayout(cardLayout);
        
        // initialize the chart
        initializeChart();
        
        // no data available view
        noDataLabel = new JLabel("");
        add(noDataLabel, NO_DATA_CARD);
        cardLayout.show(this, NO_DATA_CARD);
    }
    
    /** Sets up the chart used to display the data. */
    private void initializeChart() {
    	// create a dummy chart and create the panel
    	chartPanel = new JPanel();
        //add(chartPanel, CHART_CARD);
    }
    
    /**
     * Create a chart for a given module.
     * 
     * @param dataService
     * @param module
     * 
     * @return a chart to be displayed
     */
    protected abstract JPanel createChart(DataService dataService, Module module);
    
}
